import {Language, LanguageStatusMessageMap} from '../../../common';
import {MiddlewareResponseError, Response} from '../../common';

interface PlainResponseParams<T = unknown, U extends NodeJS.EventEmitter = NodeJS.EventEmitter> extends Response {
	body?: T;
	res: U;
}

interface HttpMiddlewareErrorParams<T = unknown, U extends NodeJS.EventEmitter = NodeJS.EventEmitter> extends Omit<PlainResponseParams<T, U>, 'statusMessage'> {
	cause?: unknown
}

export class ErrorPlainResponse<T = unknown, U extends NodeJS.EventEmitter = NodeJS.EventEmitter> extends MiddlewareResponseError implements PlainResponseParams<T, U> {
	readonly body?: T;
	readonly res: U;

	constructor(statusMessage: keyof Language['statusMessages'], params: HttpMiddlewareErrorParams<T, U>) {
		super(statusMessage, params);
		this.body = params.body;
		this.res = params.res;
		this.res.emit('response', this);
		this.res.emit('close');
	}
}

export class PlainResponse<T = unknown> implements Response {
	readonly statusCode: Response['statusCode'];

	readonly statusMessage?: keyof LanguageStatusMessageMap;

	readonly headers: Response['headers'];

	readonly body?: T;

	constructor(args: PlainResponseParams<T>) {
		this.statusCode = args.statusCode;
		this.statusMessage = args.statusMessage;
		this.headers = args.headers;
		this.body = args.body;
		args.res.emit('response', this);
		args.res.emit('close');
	}
}

// TODO stream response
